/* Simple grammar for parsing Magnolia Freemarker Templates. */
grammar Freemarker;

options {
    language=Java;
    output=AST;
    ASTLabelType=CommonTree;
    k=2;
    backtrack=true;
    memoize=true;
} 

@lexer::header {
package info.magnolia.migration.task.templates.freemarker;
}

@lexer::members {
    // status flags
    boolean tagStart = false;
    boolean tagNamed = false;
    boolean commentStart = false;
    boolean tagInt = false;
    boolean sqTag = false;
    
    // counting inner (dummy) brackets
    int inBrackets = 0;
    int inInterpolations = 0;
    
    @Override
    public void recover(RecognitionException e) { throw new RuntimeException(e); }
}

@parser::header {
package info.magnolia.migration.task.templates.freemarker;
}

/**
  * Parser rules
  */
document
	: ( tag | flattext | interpolation | comment)* EOF
	;
	
tag
	: (TAG_OPEN | TAG_OPEN_SQ) tagname ' '? (attributes)* (TAG_CLOSE | TAG_CLOSE_SQ)
	;

comment
	: (TAG_OPEN | TAG_OPEN_SQ) COMMENT (TAG_CLOSE | TAG_CLOSE_SQ)
	;
	
expression
	: EXPRESSION
	;

interpolation 
	: INTERPOLATION_OPEN (expression interpolation?)* INTERPOLATION_CLOSE
	;

attributes 
	: (INTAG_DATA | INTAG_DATA_SQ)
	| interpolation
	| INTAG_OPEN! COMMENT INTAG_CLOSE!
	| INTAG_OPEN (attributes)+ INTAG_CLOSE
	;

flattext 
	: FLATTEXT
	;

tagname	: TAGNAME | SPECIAL_SLASH? interpolation
	;
	
/**
  * Lexer rules
  */
TAG_OPEN 
	:  { !tagStart && (inInterpolations == 0) && !commentStart}?=> ('<') {tagStart = true; tagNamed = false; sqTag = false;}
	;

TAG_OPEN_SQ
	:  { !tagStart && (inInterpolations == 0) && !commentStart}?=> ('[') {tagStart = true; tagNamed = false; sqTag = true;}
	;

TAG_CLOSE 
	:  { !sqTag && tagStart && inBrackets == 0 }?=> ('>') { tagStart = false; tagNamed = false;}
	;

TAG_CLOSE_SQ
	:  { sqTag && tagStart && inBrackets == 0 }?=> (']') { tagStart = false; tagNamed = false;}
	;

// just a quick hack. needs something more robust
COMMENT
	: COMMENT_OPEN COMMENT_DATA (CDASH COMMENT_DATA)* COMMENT_CLOSE
	;

fragment
COMMENT_OPEN
	: {tagStart && !commentStart && (inInterpolations == 0)}?=>('#' | '!') '--' ('-')* {commentStart = true;}
	; 
	
fragment
COMMENT_DATA
	: { commentStart}?=> ( ~('-') )+
	;

fragment
COMMENT_CLOSE
	: {commentStart}?=> ('-')* '--' {commentStart = false;}
	;
	
fragment
CDASH 	: {commentStart}?=>'-'?
	;


INTAG_OPEN 
	: { tagStart }?=> '[' { inBrackets++; }
	;

INTAG_CLOSE 
	: { tagStart &&  inBrackets > 0 }?=>']' { inBrackets--; }
	;

TAGNAME 
	: { inInterpolations < 1 && !tagNamed && tagStart }?=>('/' | '#' | '/#' | '@' | '/@')? ('a'..'z' | 'A'..'Z' | '0'..'9' | '.' | '!' )+ {tagNamed = true;}
	;


INTERPOLATION_OPEN 
	: '${'
	{ 
	 inInterpolations++;
	 if (tagStart && !tagNamed)
	   tagInt = true;
	};

INTERPOLATION_CLOSE 
	: { inInterpolations > 0 }?=>'}' 
	{ 
	  inInterpolations--; 
	  if (tagInt)
	  { 
	    tagInt = false; tagNamed = true;
	  } 
	};
	
INTAG_DATA 
	: { !sqTag && tagNamed && (inInterpolations == 0) && !commentStart}?=> ( ~('<' | '>' | '[' | ']' | '$' ) )+
	;

INTAG_DATA_SQ
	: { sqTag && tagNamed && (inInterpolations == 0) && !commentStart}?=> ( ~('[' | ']' | '$' ) )+
	;

FLATTEXT 
	: { !tagStart && (inInterpolations == 0) && !commentStart}?=> ( ~( '<' | '>' | '[' | ']' | '$' ) )+ | '$'
	;

EXPRESSION
	: { inInterpolations > 0}?=>( ~( '$' | '}') )+
	;

//
SPECIAL_SLASH
	:	'/'
	;