/* Simple grammar for parsing Magnolia JSP Templates. */
grammar Jsp;

options {
    language=Java;
    output=AST;
    ASTLabelType=CommonTree;
    backtrack=true;
    memoize=true;
} 

@lexer::header {
package info.magnolia.migration.task.templates.jsp;
}

@lexer::members {
    // status flags
    boolean tagStart = false;
    boolean tagNamed = false;
    boolean scriptletStart = false;
    
    // counting inner (dummy) brackets
    int inBrackets = 0;
    int inInterpolations = 0;


    @Override
    public void recover(RecognitionException e) { throw new RuntimeException(e); }
}

@parser::header {
package info.magnolia.migration.task.templates.jsp;
}

/**
  * Parser rules
  */
document
	: ( scriptlet | flattext | tag | interpolation)* EOF
	;
	
tag
	: TAG_OPEN tagname (attributes)* TAG_CLOSE
	;

interpolation 
	: INTERPOLATION_OPEN (EXPRESSION interpolation?)+ INTERPOLATION_CLOSE
	;

scriptlet
	: SCRIPTLET_OPEN (javacode)+ SCRIPTLET_CLOSE
	;


attributes
	: scriptlet
	| INTAG_DATA
	| interpolation
	| INTAG_OPEN ( INTAG_DATA | interpolation )* INTAG_CLOSE
	;

expression
	: EXPRESSION
	;

javacode 
	: JAVACODE
	;

flattext 
	: FLATTEXT
	;

tagname	: TAGNAME
	;
/**
  * Lexer rules
  */
TAG_OPEN 
	:  { !tagStart && (inInterpolations == 0) }?=> '<' {tagStart = true;}
	;

TAG_CLOSE 
	:  { tagStart && inBrackets == 0 }?=> '>' { tagStart = false; tagNamed = false; }
	;

INTAG_OPEN 
	: { tagNamed }?=> '<' { inBrackets++; }
	;

INTAG_CLOSE 
	: { tagNamed &&  inBrackets > 0 }?=>'>' { inBrackets--; }
	;

TAGNAME 
	: { !tagNamed && tagStart }?=>( '/' )? ( ~( '<' | '>' | ' ' | '/' ) )+ {tagNamed = true; }
	;

INTERPOLATION_OPEN 
	:  '${' { inInterpolations++; }
	;

INTERPOLATION_CLOSE
	: { inInterpolations > 0}?=>'}' { inInterpolations--; }
	;

INTAG_DATA 
	: { tagNamed && (inInterpolations == 0) }?=> ( ~( '<' | '>' | '$' ) )+ 
	;

JAVACODE 
	: { scriptletStart }?=> ( ~('%') )+ | '%'
	;

FLATTEXT 
	: { !tagStart && (inInterpolations == 0) && !scriptletStart }?=> ( ~( '<' | '>' | '$' | '}' ) )+ 
	;


EXPRESSION 
	: { inInterpolations > 0 }?=> ( ~( '$' |  '}' ) )+
	;

SCRIPTLET_OPEN
	: '<%' { scriptletStart = true; }
	;

SCRIPTLET_CLOSE
	: { scriptletStart }?=> '%>' { scriptletStart = false; }
	;
