package scripts.migration.from4_4To4_5.export;

import info.magnolia.cms.core.Content
import info.magnolia.cms.core.HierarchyManager
import info.magnolia.cms.core.ItemType
import info.magnolia.cms.core.Path
import info.magnolia.cms.core.Content.ContentFilter
import info.magnolia.cms.util.ContentUtil
import info.magnolia.context.Context
import info.magnolia.importexport.BootstrapUtil
import info.magnolia.module.admininterface.pages.DevelopmentUtilsPage

import org.apache.commons.lang.StringUtils

class ExportConfiguration {

    Context ctx
    HierarchyManager hm

    Collection toExport;
    String targetDirectory;


    /**
     * This method allows you to export any configuration within the defined modules.
     *
     * @param moduleExport A list of all modules to export
     * @param targetDirectory The target directory where to export to.
     * @param subFolders Is optional. Defining sub folders of a module to export. If Null it will export all existing sub folders.
     */
    def exportModule(Collection moduleExport, String targetDirectory, Collection subFolders) {
        this.toExport = moduleExport;
        this.targetDirectory = targetDirectory;
        // Default workspace dor module export
        String workspace = "config";

        toExport.each{ String moduleName ->

            String modulePath = "/modules/"+moduleName
            if(subFolders != null){
                subFolders.each{String subFolder ->
                    exportConfigFolder(modulePath +"/"+ subFolder, null, workspace)
                }
            } else {
                exportConfigFolder(modulePath, null, workspace)
            }
        }
    }

    /**
     * This method allows you to export any site definition.
     *
     * @param websiteToExport A list of all sites to export
     * @param targetDirectory The target directory where to export to.
     * @param subFolders Is optional. Defining sub folders of a site to export. If Null it will export all existing sub folders.
     */
    def exportWebsite(Collection websiteToExport, String targetDirectory, Collection subFolders) {
        this.toExport = websiteToExport;
        this.targetDirectory = targetDirectory;
        // Default workspace dor module export
        String workspace = "website";

        toExport.each{ String name ->

            String modulePath = "/"+name
            if(subFolders != null){
                subFolders.each{String subFolder ->
                    exportWebsiteFolder(modulePath +"/"+ subFolder, null, workspace)
                }
            } else {
                exportWebsiteFolder(modulePath, null, workspace)
            }
        }

    }


    /**
     * Exports all nodes within the passed folder.
     * You can either pass a path to a folder or directly the folder node itself.
     *
     * @param folderPath Path to the folder node which should be processed.
     * @param folder The node of the folder which should be processed.
     * @param workspace The name of the workspace which should be processed.
     */
    def exportWebsiteFolder(String folderPath, Content folder, String workspace) {
        Content folderToExport
        if(folderPath != null){
            hm = ctx.getHierarchyManager(workspace)
            folderToExport = hm.getContent(folderPath)
        } else{
            folderToExport = folder;
        }
        if(folderToExport.hasChildren(ItemType.CONTENTNODE.toString())){
            println "ready to export"
            println folderToExport.getHandle()
            println "to "+getLocalDir(folderToExport.getHandle())
            println "*************"

            File directory = new File(Path.getAbsoluteFileSystemPath(getLocalDir(folderToExport.getHandle())));
            directory.mkdirs();
            BootstrapUtil.export(folderToExport, directory);

        }
    }


    /**
     * Exports all configuration nodes within the passed folder and all its sub folders.
     * You can either pass a path to a folder or directly the folder node itself.
     *
     * @param folderPath Path to the folder node which should be processed.
     * @param folder The node of the folder which should be processed.
     * @param workspace The name of the workspace which should be processed.
     */
    def exportConfigFolder(String folderPath, Content folder, String workspace) {
        Content folderToExport
        //Exclude MetaData
        ContentFilter chieldFilter = ContentUtil.EXCLUDE_META_DATA_CONTENT_FILTER;
        if(folderPath != null){
            hm = ctx.getHierarchyManager(workspace)
            folderToExport = hm.getContent(folderPath)
        } else{
            folderToExport = folder;
        }
        if(folderToExport.hasChildren(ItemType.CONTENTNODE.toString())){
            println "ready to export"
            println folderToExport.getHandle()
            println "to "+getLocalDir(folderToExport.getHandle())
            println "*************"
            DevelopmentUtilsPage p = new DevelopmentUtilsPage(null, null, null);
            p.setRepository(workspace)
            p.setRootdir(getLocalDir(folderToExport.getHandle()))
            p.setParentpath(folderToExport.getHandle())
            p.setChieldFilter(chieldFilter);
            p.backupChildren();

            // create necessary parent directories
        }
        if(folderToExport.hasChildren(ItemType.CONTENT.toString())){
            Collection folders = folderToExport.getChildren(ItemType.CONTENT.toString())
            folders.each{ Content subFolder ->
                exportConfigFolder(null, subFolder, workspace)
            }
        } else if (folderToExport.isNodeType(ItemType.CONTENT.toString())){
            println "ready to export"
            println folderToExport.getHandle()
            println "to "+getLocalDir(folderToExport.getHandle())
            println "*************"
            DevelopmentUtilsPage p = new DevelopmentUtilsPage(null, null, null);
            p.setRepository(workspace)
            p.setRootdir(getLocalDir(folderToExport.getHandle()))
            p.setParentpath(folderToExport.getHandle())
            p.setChieldFilter(chieldFilter);
            p.backupChildren();
        }
    }

    def private getLocalDir(String folderHandle) {
        return targetDirectory + StringUtils.substringAfter(folderHandle,"modules")
    }
}